"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fluent_1 = require("@berun/fluent");
const PluginPreset_1 = require("./PluginPreset");
class Babel extends fluent_1.FluentMap {
    constructor(parent = null, name = null) {
        super(parent, name);
        this.babelrc = fluent_1.FluentValue();
        this.root = fluent_1.FluentValue();
        this.ast = fluent_1.FluentValue();
        this.auxiliaryCommentAfter = fluent_1.FluentValue();
        this.auxiliaryCommentBefore = fluent_1.FluentValue();
        this.babelrcRoots = fluent_1.FluentValue();
        this.code = fluent_1.FluentValue();
        this.comments = fluent_1.FluentValue();
        this.compact = fluent_1.FluentValue();
        this.configFile = fluent_1.FluentValue();
        this.cwd = fluent_1.FluentValue();
        this.envName = fluent_1.FluentValue();
        this.extends = fluent_1.FluentValue();
        this.filename = fluent_1.FluentValue();
        this.filenameRelative = fluent_1.FluentValue();
        this.getModuleId = fluent_1.FluentValue();
        this.highlightCode = fluent_1.FluentValue();
        this.ignore = fluent_1.FluentValue();
        this.inputSourceMap = fluent_1.FluentValue();
        this.minified = fluent_1.FluentValue();
        this.moduleId = fluent_1.FluentValue();
        this.moduleIds = fluent_1.FluentValue();
        this.moduleRoot = fluent_1.FluentValue();
        this.only = fluent_1.FluentValue();
        this.retainLines = fluent_1.FluentValue();
        this.shouldPrintComment = fluent_1.FluentValue();
        this.sourceFileName = fluent_1.FluentValue();
        this.sourceMaps = fluent_1.FluentValue();
        this.sourceRoot = fluent_1.FluentValue();
        this.sourceType = fluent_1.FluentValue();
        this.wrapPluginVisitorMethod = fluent_1.FluentValue();
        this.plugins = new fluent_1.FluentMap(this);
        this.presets = new fluent_1.FluentMap(this);
        this.environments = new fluent_1.FluentMap(this);
        this.extendfluent();
    }
    env(name) {
        if (!this.environments.has(name)) {
            this.environments.set(name, new Babel(this, name));
        }
        return this.environments.get(name);
    }
    plugin(name, opts) {
        if (!this.plugins.has(name)) {
            this.plugins.set(name, new PluginPreset_1.PluginPreset(this, name, opts));
        }
        return this.plugins.get(name);
    }
    preset(name, opts) {
        if (!this.presets.has(name)) {
            this.presets.set(name, new PluginPreset_1.PluginPreset(this, name, opts));
        }
        return this.presets.get(name);
    }
    toConfig(omit = []) {
        return Object.assign(super.toConfig(omit.concat(['plugins', 'presets', 'environments'])) || {}, this.clean({
            env: this.environments.values().reduce((accum, p) => {
                accum[p.name] = p.toConfig();
                return accum;
            }, {}),
            plugins: this.plugins.values().map(p => p.toConfig()),
            presets: this.presets.values().map(p => p.toConfig())
        }));
    }
    merge(obj, omit = []) {
        if (!obj)
            return this;
        if (!omit.includes('plugins') && 'plugins' in obj) {
            obj.plugins.forEach(p => {
                const name = Array.isArray(p) ? p[0] : p;
                this.plugin(name).merge(p);
            });
        }
        if (!omit.includes('presets') && 'presets' in obj) {
            obj.presets.forEach(p => {
                const name = Array.isArray(p) ? p[0] : p;
                this.preset(name).merge(p);
            });
        }
        if (!omit.includes('env') && 'env' in obj) {
            Object.keys(obj.environments).forEach(name => this.env(name).merge(obj.env[name]));
        }
        return super.merge(obj, [...omit, 'plugins', 'presets', 'env']);
    }
}
exports.Babel = Babel;
